@@ -1,22 +1,32 @@
 /* SNMP reader plugin for GKrellM 
-|  Copyright (C) 2000-2003  Christian W. Zuckschwerdt <zany@triq.net>
+|  Copyright (C) 2000-2009  Christian W. Zuckschwerdt <zany@triq.net>
 |
 |  Author:  Christian W. Zuckschwerdt  <zany@triq.net>  http://triq.net/
 |  Latest versions might be found at:  http://gkrellm.net/
 |
-| This program is free software; you can redistribute it and/or
-| modify it under the terms of the GNU General Public License
-| as published by the Free Software Foundation; either version 2
-| of the License, or (at your option) any later version.
+| GKrellM_SNMP is free software; you can redistribute it and/or
+| modify it under the terms of the GNU General Public License as
+| published by the Free Software Foundation; either version 2 of
+| the License, or (at your option) any later version.
 |
+| In addition, as a special exception, the copyright holders give
+| permission to link the code of this program with the OpenSSL library,
+| and distribute linked combinations including the two.
+| You must obey the GNU General Public License in all respects
+| for all of the code used other than OpenSSL.  If you modify
+| file(s) with this exception, you may extend this exception to your
+| version of the file(s), but you are not obligated to do so.  If you
+| do not wish to do so, delete this exception statement from your
+| version.  If you delete this exception statement from all source
+| files in the program, then also delete it here.
+
 | This program is distributed in the hope that it will be useful,
 | but WITHOUT ANY WARRANTY; without even the implied warranty of
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 | GNU General Public License for more details.
-|
+
 | You should have received a copy of the GNU General Public License
-| along with this program; if not, write to the Free Software
-| Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+| along with GKrellM_SNMP. If not, see <http://www.gnu.org/>.
 */
 
 /* Installation:
@@ -33,15 +43,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 
-#ifdef NETSNMP
-#include <net-snmp/net-snmp-config.h>
-#include <net-snmp/net-snmp-includes.h>
-#define RECEIVED_MESSAGE NETSNMP_CALLBACK_OP_RECEIVED_MESSAGE
-#define TIMED_OUT NETSNMP_CALLBACK_OP_TIMED_OUT
-#ifdef DEBUG_SNMP
-#include <net-snmp/snmp_debug.h>
-#endif /* DEBUG_SNMP */
-#else /* UCD-SNMP */
+#ifdef UCDSNMP
 #include <ucd-snmp/asn1.h>
 #include <ucd-snmp/mib.h>
 #include <ucd-snmp/parse.h>
@@ -52,7 +54,15 @@
 #ifdef DEBUG_SNMP
 #include <ucd-snmp/snmp_debug.h>
 #endif /* DEBUG_SNMP */
-#endif /* UCD-SNMP */
+#else /* UCDSNMP */
+#include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-includes.h>
+#define RECEIVED_MESSAGE NETSNMP_CALLBACK_OP_RECEIVED_MESSAGE
+#define TIMED_OUT NETSNMP_CALLBACK_OP_TIMED_OUT
+#ifdef DEBUG_SNMP
+#include <net-snmp/snmp_debug.h>
+#endif /* DEBUG_SNMP */
+#endif /* UCDSNMP */
 
 #include <sys/time.h>
 
@@ -607,6 +617,29 @@ simpleSNMPsend(struct snmp_session *session,
 }
 
 
+/* GKrellM Callbacks */
+
+static void
+cb_draw_chart(gpointer data)
+{
+	Reader *reader = (Reader *)data;
+
+	gchar *text = render_label(reader);
+	gkrellm_draw_chartdata(reader->chart);
+	gkrellm_draw_chart_text(reader->chart,
+				DEFAULT_STYLE_ID,
+				text);
+	gkrellm_draw_chart_to_screen(reader->chart);
+	g_free(text);
+}
+
+static void
+cb_chart_click(GtkWidget *widget, GdkEventButton *event, gpointer data)
+	{
+		if (event->button == 3)
+			gkrellm_chartconfig_window_create(data);
+	}
+
 /* GKrellM interface */
  
 static GkrellmMonitor *mon;
@@ -681,13 +714,7 @@ update_plugin()
 		( (reader->divisor == 0) ? 1 : reader->divisor );
 		
 				    gkrellm_store_chartdata(reader->chart, 0, val);
-				    text = render_label(reader);
-				    gkrellm_draw_chartdata(reader->chart);
-				    gkrellm_draw_chart_text(reader->chart,
-							    DEFAULT_STYLE_ID,
-							    text);
-				    gkrellm_draw_chart_to_screen(reader->chart);
-				    g_free(text);
+				    cb_draw_chart(reader);
 
 				    text = render_info(reader);
 				    gtk_tooltips_set_tip(reader->tooltip, reader->chart->drawing_area, text, "");
@@ -780,13 +807,6 @@ chart_expose_event(GtkWidget *widget, GdkEventExpose *ev)
     return FALSE;
 }
 
-static void
-cb_chart_click(GtkWidget *widget, GdkEventButton *event, gpointer data)
-	{
-		if (event->button == 3)
-			gkrellm_chartconfig_window_create(data);
-	}
-
 static void
 create_chart(GtkWidget *vbox, Reader *reader, gint first_create)
 {
@@ -797,6 +817,10 @@ create_chart(GtkWidget *vbox, Reader *reader, gint first_create)
 
     gkrellm_chart_create(vbox, mon, reader->chart, &reader->chart_config);
 
+    gkrellm_chartconfig_grid_resolution_adjustment(reader->chart_config,
+                /*map*/TRUE, /*spin_factor*/1.0, /*low*/1, /*high*/100000000,
+			/*step0*/0, /*step1*/0, /*digits*/0, /*width*/50);
+
     reader->chart_data = gkrellm_add_default_chartdata(reader->chart, "Plugin Data");
     
     gkrellm_monotonic_chartdata(reader->chart_data, FALSE);
@@ -808,6 +832,7 @@ create_chart(GtkWidget *vbox, Reader *reader, gint first_create)
 
     if (first_create)
     {
+            gkrellm_set_draw_chart_function(reader->chart, cb_draw_chart, reader);
 	    gtk_signal_connect(GTK_OBJECT(reader->chart->drawing_area),
 			       "expose_event", (GtkSignalFunc) chart_expose_event, NULL);
 	    gtk_signal_connect(GTK_OBJECT(reader->chart->drawing_area),
@@ -974,33 +999,68 @@ save_plugin_config(FILE *f)
 	      reader->oid_str, unit,
 	      reader->delay, reader->delta,
 	      reader->divisor, reader->scale);
+      gkrellm_save_chartconfig(f, reader->chart_config, PLUGIN_CONFIG_KEYWORD, label);
       g_free(label);
       g_free(unit);
   }
 }
 
 static void
-load_plugin_config(gchar *arg)
+load_plugin_config(gchar *config_line)
 {
-  Reader *reader, *nreader;
+  Reader *reader, *nreader = NULL;
 
   gchar   proto[CFG_BUFSIZE], bufl[CFG_BUFSIZE];
   gchar   bufc[CFG_BUFSIZE], bufp[CFG_BUFSIZE];
   gchar   bufo[CFG_BUFSIZE], bufu[CFG_BUFSIZE];
+  gchar   buft[CFG_BUFSIZE], peer[CFG_BUFSIZE];
   gint    n;
 
+  if (sscanf(config_line, GKRELLM_CHARTCONFIG_KEYWORD " %s %[^\n]", bufl, bufc) == 2) {
+	g_strdelimit(bufl, "_", ' ');
+	/* look for any such reader */
+	for (reader = readers; reader ; reader = reader->next) {
+		if (!strcmp(reader->label, bufl)) {
+			nreader = reader;
+			break;
+		}
+	}
+	/* look for unconf'd reader */
+	for (reader = readers; reader ; reader = reader->next) {
+		if (!strcmp(reader->label, bufl) && !reader->chart_config) {
+			nreader = reader;
+			break;
+		}
+	}
+	if (!nreader) {/* well... */
+		fprintf(stderr, "chart_config appeared before chart, this isn't handled\n%s\n", config_line);
+		return;
+	}
+	//"chart_config in "
+	gkrellm_load_chartconfig(&nreader->chart_config, bufc, /*max_cd*/1);
+  	return;
+  }
+  
   reader = g_new0(Reader, 1); 
 
-  n = sscanf(arg, "%s %[^:]://%[^@]@%[^:]:%d/%s %s %d %d %d %d",
-	     bufl, proto, bufc, bufp, &reader->port, bufo, bufu,
+  n = sscanf(config_line, "%s %[^:]://%[^@]@%[^:]:%[^:]:%d/%s %s %d %d %d %d",
+	     bufl, proto, bufc, buft, bufp, &reader->port, bufo, bufu,
 	     &reader->delay, &reader->delta,
 	     &reader->divisor, &reader->scale);
-  if (n >= 6)
+  if (n >= 6) {
+	g_snprintf(peer, CFG_BUFSIZE, "%s:%s", buft, bufp);
+	peer[CFG_BUFSIZE-1] = '\0';
+  } else
+	  n = sscanf(config_line, "%s %[^:]://%[^@]@%[^:]:%d/%s %s %d %d %d %d",
+	     bufl, proto, bufc, peer, &reader->port, bufo, bufu,
+	     &reader->delay, &reader->delta,
+	     &reader->divisor, &reader->scale);
+  if (n >= 7)
     {
       if (g_strcasecmp(proto, "snmp") == 0) {
 	gkrellm_dup_string(&reader->label, bufl);
 	gkrellm_dup_string(&reader->community, bufc);
-	gkrellm_dup_string(&reader->peer, bufp);
+	gkrellm_dup_string(&reader->peer, peer);
 	if (reader->delay < 10)
 	    reader->delay = 100;
 	if (reader->divisor == 0)
@@ -1304,17 +1364,17 @@ static gchar    *plugin_info_text =
 "It's measured in GKrellM ticks -- that's 1/10 seconds.\n"
 "Label is a unique name that gets prepended to your reader.\n"
 "Unit is just a string thats appended to your reader.\n"
+"You can prepend a specific transport to the peer name.\n"
+"(i.e. tcp:192.168.0.1)\n"
 "\n"
 "Some examples:\n"
 "\n"
 "(1)\n"
-"The ambiente temperature sensor for Oldenburg i.O., Germany\n"
-" (see http://www.PMNET.uni-oldenburg.de/temperatur.php)\n"
-"is world readable using the following community/server, oid\n"
-"public / 134.106.172.2 port 161 oid .1.3.6.1.4.1.2021.8.1.101.1\n"
+"The ambiente temperature sensor for some net-snmp server\n"
+"public / 192.168.1.2 port 161 oid .1.3.6.1.4.1.2021.8.1.101.1\n"
 "\n"
 "That is:\n"
-" SNMP peer '134.106.172.2' (kyle.pmnet.uni-oldenburg.de)\n"
+" SNMP peer '192.168.1.2' (some.server.name)\n"
 " SNMP port '161' (that's the default)\n"
 " SNMP community name 'public'\n"
 " SNMP oid '.1.3.6.1.4.1.2021.8.1.101.1'\n"
@@ -1325,26 +1385,26 @@ static gchar    *plugin_info_text =
 "\n"
 "Server CPU load using a string ranging from 0.00 to 1.00\n"
 "\n"
-"public / 134.106.120.1 pot 161 oid .1.3.6.1.4.1.2021.10.1.3.1\n"
-"(Thats the load factor for PMNET's Stan)\n"
+"public / 192.168.1.3 pot 161 oid .1.3.6.1.4.1.2021.10.1.3.1\n"
+"(Thats the load factor for some server)\n"
 "\n"
 "(3)\n"
 "\n"
 "Server CPU load using integer variable ranging from 0 to 100\n"
 "\n"
-"public / 134.106.172.2 port 161 oid .1.3.6.1.4.1.2021.10.1.5.1\n"
-"(Thats the percentile load on PMNET's Kyle)\n"
+"public / 192.168.1.4 port 161 oid .1.3.6.1.4.1.2021.10.1.5.1\n"
+"(Thats the percentile load for some server)\n"
 "\n"
 "please mail any problems/questions to me...\n"
 ;
 
 static gchar    *plugin_about_text =
-   "SNMP plugin 0.20\n"
+   "SNMP plugin 1.0\n"
    "GKrellM SNMP monitor Plugin\n\n"
-   "Copyright (C) 2000-2003 Christian W. Zuckschwerdt <zany@triq.net>\n"
+   "Copyright (C) 2000-2006 Christian W. Zuckschwerdt <zany@triq.net>\n"
    "\n"
    "http://triq.net/gkrellm.html\n\n"
-   "Released under the GNU Public Licence"
+   "Released under the GNU Public Licence with OpenSSL exemption"
 ;
 
 static gchar *reader_title[CLIST_WIDTH] =
